;; -*- no-byte-compile: t; lexical-binding: nil -*-
(define-package "lsp-treemacs" "20260515.746"
  "LSP treemacs."
  '((emacs    "29.1")
    (dash     "2.18.0")
    (f        "0.20.0")
    (ht       "2.0")
    (treemacs "2.5")
    (lsp-mode "6.0"))
  :url "https://github.com/emacs-lsp/lsp-treemacs"
  :commit "3519ac907ea391e18d9599375b116aeeb6f8a38a"
  :revdesc "3519ac907ea3"
  :keywords '("languages"))
